/**
 *  \file i2c.h
 *
 *  \date Created on: 6-feb-2009
 *  \author Author: Claudio Salvadori and Christian Nastasi
 *  \brief I2C communication files
 *
 */

#ifndef _i2c_h_
#define _i2c_h_

#include "ee.h"

#define MAX_RETRY	 1000

/******************************************************************************/
/*                            Type Definitions                                */
/******************************************************************************/
/**
* @brief I2C exit status
*/
typedef enum {
	I2C_FAILURE = 0,	/**< General error code.  */
	I2C_SUCCESS = 1,	/**< Success, no error occurs. */
} i2c_status_t;

//******************************************************************************/
/*                            INLINE Functions                                */
/******************************************************************************/
/* __INLINE__ ... */

/******************************************************************************/
/*                         Function Delcarations                              */
/******************************************************************************/
/**
* @brief Initialize the I2C Driver
*
*
* @return 	Return \ref I2C_SUCCESS if no error occurs,
* 		otherwise a specific error code.
*/
i2c_status_t
i2c_init(void);

/**
* @brief Read a byte
*
* @param device 	The device ID (device address).
* @param address 	The device register (register address).
* @param data 		A pointer to store the read byte.
*
* @return 	Return \ref I2C_SUCCESS if no error occurs,
* 		otherwise a specific error code.
*/
i2c_status_t
i2c_read_byte(	unsigned char device,
		unsigned char address,
		unsigned char *data
);

/**
* @brief Write a byte
*
* @param device		The device ID (device address).
* @param address 	The device register (register address).
* @param data 		The byte to write.
*
* @return 	Return \ref I2C_SUCCESS if no error occurs,
*		otherwise a specific error code.
*/
i2c_status_t
i2c_write_byte(	unsigned char device,
		unsigned char address,
		unsigned char data
);



#endif
